@@ -53,7 +53,7 @@ targets_shlib = list()
 
 flag_blacklist = ['-Werror', '-U_FORTIFY_SOURCE', '-m32', '-m64', '-Wno-#pragma-messages']
 
-@lru_cache
+@lru_cache(maxsize=None)
 def detect_arch():
     if 'DEB_HOST_ARCH' in os.environ:
         return os.environ['DEB_HOST_ARCH']
@@ -82,6 +82,13 @@ def multilib():
     else:
         return 'lib32'
 
+@lru_cache(maxsize=None)
+def detect_multiarch():
+    if 'DEB_HOST_MULTIARCH' in os.environ:
+        return os.environ['DEB_HOST_MULTIARCH']
+    else:
+        return dpkg_architecture('-q', 'DEB_HOST_MULTIARCH').rstrip()
+
 def collect_defaults(args):
     global defaults
     def_cxxflags = []
@@ -124,7 +131,7 @@ def cc_defaults(**args):
     cflags = filter_flags(def_cflags + args.get('cflags', []) + [f"-I{rel(inc)}" for inc in local_include_dirs])
     ldflags = filter_flags(def_ldflags + args.get('ldflags', []))
     ldlibs = def_ldlibs + [f"-l{lib[3:]}" for lib in args.get('shared_libs', [])]
-    srcs = [f"{rel(path)}" for path in def_srcs + args.get('srcs', [])]
+    srcs = [(f"{rel(path)}" if not path.startswith('$') else path) for path in def_srcs + args.get('srcs', [])]
     print_vars(args['name'], locals(), ['cxxflags', 'cflags', 'ldflags', 'ldlibs', 'srcs'])
     print()
 
@@ -155,10 +162,20 @@ def cc_compile_link(args, binary: bool = True, shared: bool = False, variables:
     shared_libs += [lib for lib in args.get('whole_static_libs', []) if lib not in all_modules]
     ldlibs = def_ldlibs + [f"-l{lib[3:]}" for lib in shared_libs]
     shlibdeps = []
+
+    multiarch_libdir = Path(f"/usr/lib/{detect_multiarch()}")
+    multiarch_android_libdir = multiarch_libdir / "android"
+
+    external_android_shlibs = [lib for lib in shared_libs if (lib not in all_modules) and (multiarch_android_libdir / f"{lib}.so").exists()]
+    if external_android_shlibs:
+        ldflags += [f"-L{multiarch_android_libdir}"]
+    external_shlibs = external_android_shlibs + [lib for lib in shared_libs if (lib not in all_modules) and (multiarch_libdir / f"{lib}.so").exists()]
+
     for lib in shared_libs:
         cxxflags += [f"$({lib}_INCLUDES)", f"$({lib}_SYSTEM_INCLUDES)"]
         cflags += [f"$({lib}_INCLUDES)", f"$({lib}_SYSTEM_INCLUDES)"]
-        shlibdeps += [f"{lib}.so"]
+        if lib not in external_shlibs:
+            shlibdeps += [f"{lib}.so"]
 
     if any(lib for lib in shared_libs if lib in all_modules):
         ldflags += ["-L."]
@@ -168,7 +185,7 @@ def cc_compile_link(args, binary: bool = True, shared: bool = False, variables:
         static_libs += ["-Wl,--whole-archive"]
         static_libs += [f"{lib}.a" for lib in args.get('whole_static_libs', []) if lib in all_modules]
         static_libs += ["-Wl,--no-whole-archive"]
-    srcs = [f"{rel(path)}" for path in def_srcs + args.get('srcs', [])]
+    srcs = [(f"{rel(path)}" if not path.startswith('$') else path) for path in def_srcs + args.get('srcs', [])]
     if variables:
         print(f"{args['name']}_DIR        = {current_recipe['dir']}")
         if shared:
